@@ -2,7 +2,7 @@
 # This marvelous code is Public Domain.
 #
 
-from typing import Any, Dict, Set, List, Union
+from typing import Any, Dict, Set, List, Union, Optional
 import logging
 import functools as ft
 import aiosql as sql  # type: ignore
@@ -10,10 +10,9 @@ import aiosql as sql  # type: ignore
 log = logging.getLogger("anodb")
 
 # get package version
-__version__ = "4.0.2"
-# FIXME currently broken because of aiosql dependency on typing_extension<4
-# import pkg_resources as pkg  # type: ignore
-# __version__ = pkg.require("anodb")[0].version
+import pkg_resources as pkg  # type: ignore
+
+__version__ = pkg.require("anodb")[0].version
 
 
 #
@@ -26,14 +25,24 @@ class DB:
     and SQL execution methods from aiosql.
     """
 
-    def __init__(self, db: str, conn: str, queries: str = None,
-                 options: Union[None, str, Dict[str, Any]] = None,
-                 auto_reconnect: bool = True,
-                 debug: bool = False,
-                 **conn_options):
+    # database connection driver and variants, with a little hardcoding
+    SQLITE = ("sqlite3", "sqlite")
+    POSTGRES = ("psycopg", "pg", "postgres", "postgresql", "psycopg3")
+    # others stay as-is: psycopg2 pg8000…
+
+    def __init__(
+        self,
+        db: str,
+        conn: Optional[str],
+        queries: Optional[str] = None,
+        options: Union[None, str, Dict[str, Any]] = None,
+        auto_reconnect: bool = True,
+        debug: bool = False,
+        **conn_options,
+    ):
         """DB constructor
 
-        - db: database engine, `sqlite` or `postgres` (or `psycopg[23]`)
+        - db: database engine/driver)
         - conn: database-specific connection string
         - queries: file holding queries for `aiosql`, may be empty
         - options: database-specific options in various forms
@@ -41,29 +50,27 @@ class DB:
         - debug: debug mode generate more logs through `logging`
         - conn_options: database-specific `kwargs` constructor options
         """
+        self.__version__ = __version__
+        self.__aiosql_version__ = pkg.require("aiosql")[0].version
         log.info(f"creating DB for {db}")
-        # database connection driver, with a little hardcoding
-        SQLITE = ('sqlite3', 'sqlite')
-        POSTGRES = ('pg', 'postgres', 'postgresql', 'psycopg', 'psycopg3')
-        MYSQL = ('mysql', 'pymysql')
-        self._db = 'sqlite3' if db in SQLITE else \
-            'psycopg' if db in POSTGRES else \
-            'psycopg2' if db == 'psycopg2' else \
-            'pymysql' if db in MYSQL else \
-            None
-        assert self._db, f"database {db} is supported"
+        # this is the class name
+        self._db = (
+            "sqlite3" if db in self.SQLITE else "psycopg" if db in self.POSTGRES else db
+        ).lower()
+        assert self._db in sql.aiosql._ADAPTERS, f"database {db} is supported"
         # connection…
         self._conn = None
         self._conn_str = conn
         self._conn_options: Dict[str, Any] = {}
-        if not options:
+        if options is None:
             pass
         elif isinstance(options, str):
             import ast
+
             self._conn_options.update(ast.literal_eval(options))
         elif isinstance(options, dict):
             self._conn_options.update(options)
-        else:  # pragma: no cover
+        else:
             raise Exception(f"unexpected type for options: {type(options)}")
         self._conn_options.update(conn_options)
         # various boolean flags
@@ -106,12 +113,19 @@ class DB:
                 log.warning(f"DB {self._db} rollback failed: {rolerr}")
             # detect a connection error for psycopg[23], to attempt a
             # reconnection should more cases be handled?
-            if self._db == "psycopg" and hasattr(self._conn, 'closed') and \
-               self._conn.closed and self._auto_reconnect:
+            if (
+                self._db == "psycopg"
+                and hasattr(self._conn, "closed")
+                and self._conn.closed
+                and self._auto_reconnect
+            ):
                 self._reconn = True
-            elif self._db == "psycopg2" and \
-               hasattr(self._conn, 'closed') and \
-               self._conn.closed == 2 and self._auto_reconnect:
+            elif (
+                self._db == "psycopg2"
+                and hasattr(self._conn, "closed")
+                and self._conn.closed == 2
+                and self._auto_reconnect
+            ):
                 self._reconn = True
             # re-raise initial error
             raise error
@@ -128,43 +142,65 @@ class DB:
                 self._available_queries.add(q)
                 self._count[q] = 0
 
-    def _aiosql_driver(self):
-        if self._db == "pymysql":
-            from aiosql_mysql import PyMySQLAdaptor  # type: ignore
-            return PyMySQLAdaptor
-        else:
-            # FIXME aiosql does not know yet about psycopg?
-            return self._db if self._db != "psycopg" else "psycopg2"
-
     def add_queries_from_path(self, fn: str):
         """Load queries from a file or directory."""
-        self._create_fns(sql.from_path(fn, self._aiosql_driver()))
+        self._create_fns(sql.from_path(fn, self._db))
 
     def add_queries_from_str(self, qs: str):
         """Load queries from a string."""
-        self._create_fns(sql.from_str(qs, self._aiosql_driver()))
+        self._create_fns(sql.from_str(qs, self._db))
 
     def _connect(self):
-        """Create a database connection."""
+        # FIXME could/should be more generic?
+        """Create a database connection.
+
+        This is kind of a pain because PEP249 does not impose a unified
+        signature for connect:-(
+        """
         log.info(f"DB {self._db}: connecting")
-        if self._db == 'sqlite3':
+        # load module
+        module = self._db
+        if self._db == "sqlite3":
             import sqlite3 as db
-            self._db_vervion = db.version
-            return db.connect(self._conn_str, **self._conn_options)
-        elif self._db == 'psycopg':
+
+            self._db_version = db.version
+        # skip apsw as DB API support is really partial
+        elif self._db == "psycopg":
             import psycopg as db  # type: ignore
-            self._db_version = db.__version__
-            return db.connect(self._conn_str, **self._conn_options)
-        elif self._db == 'psycopg2':
+        elif self._db == "psycopg2":
             import psycopg2 as db  # type: ignore
-            self._db_version = db.__version__
-            return db.connect(self._conn_str, **self._conn_options)
-        elif self._db == 'pymysql':
+        elif self._db == "pg8000":
+            import pg8000 as db  # type: ignore
+        elif self._db == "pygresql":
+            import pgdb as db  # type: ignore
+
+            module = "pgdb"
+        elif self._db == "pymysql":
             import pymysql as db  # type: ignore
-            return db.connect(self._conn_str, **self._conn_options)
+        elif self._db in ("MySQLdb", "mysqldb"):
+            import MySQLdb as db  # type: ignore
+
+            module = "mysqlclient"
+        elif self._db in ("mysql-connector", "mysql.connector"):
+            import mysql.connector as db  # type: ignore
+
+            module = "mysql.connector"
+        elif self.db == "mariadb":  # pragma: no cover
+            import mariadb as db  # type: ignore
         else:  # pragma: no cover
-            # note: aiosql currently supports sqlite & postgres
             raise Exception(f"unexpected db {self._db}")
+        # get version if needed
+        if not hasattr(self, "_db_version"):
+            if hasattr(db, "__version__"):
+                self._db_version = db.__version__
+            else:
+                self._db_version = pkg.require(module)[0].version
+        # do connect
+        return (
+            db.connect(self._conn_str, **self._conn_options)
+            if self._conn_str
+            else db.connect(**self._conn_options)
+        )
 
     def _reconnect(self):
         """Try to reconnect to database."""
@@ -179,9 +215,10 @@ class DB:
         self._reconn = False
 
     def connect(self):
-        """Create database connection if needed."""
-        if '_conn' not in self.__dict__ or not self._conn:
+        """Create (if needed) and return the database connection."""
+        if "_conn" not in self.__dict__ or not self._conn:
             self._conn = self._connect()
+        return self._conn
 
     def cursor(self):
         """Get a cursor on the current connection."""
@@ -206,5 +243,5 @@ class DB:
         return f"connection to {self._db} database ({self._conn_str})"
 
     def __del__(self):
-        if '_conn' in self.__dict__ and self._conn:
+        if hasattr(self, "_conn") and self._conn:
             self.close()
